#!/usr/bin/perl
#
# $Id: raid.pl,v 1.3 2000/06/29 08:23:44 asun Exp $
# parses /proc/mdstat for faulty partition information
# updates Meta database and sends email to <sys@localhost>
#

# make sure /etc/raidtab exists
# present on RaQ4r but *NOT* on RaQ4i
#
exit 1 unless (-f "/etc/raidtab");

use Cobalt::Meta;
use Locale::gettext;
use POSIX;

$MDSTAT='/proc/mdstat';
$sendmail='/usr/sbin/sendmail -t -oi -O DeliveryMode=background';
$dead=0;
$previous=0;

# setup locale information
#
$language=`cat /etc/cobalt/locale`;
$language="en_US" unless ($language);
setlocale(LC_MESSAGES, $language);
bindtextdomain("swatch", "/usr/share/locale");
textdomain("swatch");

# make sure /proc/mdstat exists
#
unless (-f $MDSTAT) {
    print STDERR "no $MDSTAT (kernel doesn't have md support?)\n";
    exit 7;
}

# query database to find out current alert status
#
$Obj = new Cobalt::Meta(type => defcon);
$Obj->retrieve(raid);
if ($Obj->get(defcon) > 1) {
    $previous=1;
}
$Obj->put(name => raid, special => disk);

# parse /proc/mdstat
# searching for raid devices with bad partitions
#
open STATUS, "< $MDSTAT";
while (<STATUS>)
{
    $status .= "  $_";
    next unless /\bactive\b/;
    
    /\[(\d+)\/(\d+)\]/;
    
    if ($1 != $2) {
	next if /(?:resync|recovery)=/;
	$dead++;
	/^([a-z0-9]+)/;
	$errorlog .= "Warning: RAID device <$1> has a dead partition!\n";
    }
}
close STATUS;

if ($dead && !$previous) {
    # if raid error found
    # alert going from GREEN -> RED
    #
    $Obj->put(defcon => 3);
    $Obj->save;
    
    $subject = gettext("RAID Error detected!");
    open(ALERT, "| $sendmail") || exit 7;
    print ALERT <<END;
From: Active Monitor <root\@localhost>
To: System Alert List <sys\@localhost>
Priority: high
Subject: [Active Monitor] $subject

$errorlog
[$MDSTAT]
$status
END
    close ALERT;
}
elsif ($previous && !$dead) {
    # no raid errors found
    # error state going from RED -> GREEN
    #
    $Obj->put(defcon => 1);
    $Obj->save;
}

# exit code is 0 for normal
# and number of bad partitions on error
#
exit $dead;
