#
# $Id: Fpx.pm,v 1.34.2.2.2.2 2001/07/19 22:29:50 dtiu Exp $
#
# Written by Will DeHaan <will@cobaltnet.com>
#
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com/
#

package Cobalt::Fpx;

use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
require SelfLoader;

@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(fpx_set_web           fpx_get_web fpx_rename_web
             fpx_recurse_chmod
             fpx_rewrite_htaccess);

require File::Copy;
import File::Copy qw(move copy);
require Cobalt::User;
import Cobalt::User qw(user_exist);
require Cobalt::Vsite;
import Cobalt::Vsite qw(vsite_list vsite_get_bygroup vsite_getgroup vsite_set_fpx);
use Cobalt::Product;
use Cobalt::Util;

1;
__DATA__

sub fpx_rename_web
# rename a frontpage web site
# arguments: oldname newname
{
    my ($oldname, $newname) = @_;
    my $oldsite = $Fpx_cnfdir . '/' . $oldname . ':80.cnf';
    
    # the fpx administrative program is braindead. we rename files ourselves
    if(-f $oldsite) {
	open(NUCNF, "> $Fpx_cnfdir/${newname}:80.cnf");
	open(CFG, $oldsite);
	while(<CFG>) {
	    s/$oldname$/$newname/;
	    print NUCNF;
        }
        close(CFG);
        close(NUCNF);

        unlink($oldsite);
    }
}


sub fpx_set_web
# Enable or disable FrontPage Server Extensions 
# Arguments: group or username, 1 for on/0 for off
# Return value: a status string of the usual form form MSG_ok("2534") 
# Side effects: Calls MS's fpsrvadm.exe, modifies Apache's access.conf 
#i              via Vsite.pm
#
# Complications:
# FP requires a very lax AllowOverride All in access.conf.  This enables
# users to run CGIs with Apache's permissions.  We will only enable this
# override capability to sites with FPX enabled.  
#
# Examples: $good = Cobalt::Fpx::fpx_set_web($group,0);
#            $bad = Cobalt::Fpx::fpx_set_web($user,1);
{
  my($group,$state,$wembasterPW)=@_; # $group may also be a user

  return MSG_error("fpx_invalid_unsupported") unless ($Features{'fpx'});
  return MSG_error("fpx_cant_fpxnotavail") unless (-x $Fpx_admsrvexe);

  # Current state
  my $fpx_active = fpx_get_web($group); 
  return MSG_ok("fpx_ok_alreadyon") if ($state && $fpx_active);
  return MSG_ok("fpx_ok_alreadyoff") unless ($state || $fpx_active);
  
  # Test user vs group/site
  my($user,$could_not_install);
  if(Cobalt::User::user_exist($group))  ### User web
  {
    $user=$group; # Clean variable names
    $group='';

    my($site)=Cobalt::Vsite::vsite_getgroup($user);
    return MSG_error("fpx_invalid_baduser") unless ($site);

    # No user webs unless the site has a root web
    return MSG_error("fpx_cant_sitedisabled") unless(fpx_get_web($site));

    # fpadmsrv.exe matches $siteName in httpd.conf
    my($siteName)=(Cobalt::Vsite::vsite_get_bygroup($site))[1];

    my($user_home,$user_uid)=(getpwnam($user))[7,2];
    my($site_gid)=(getgrnam($site))[2];

    if ($state) # Enable
    {
      # Rename .htaccess files so fpadmsrv.exe doesn't die from confusion
      my(@htax) = fpx_rename_htaccess("$user_home/$Webdir");

      my($redmond)=("$Userdir/$site/$Webdir/.htaccess");
      if (-e $redmond)
      {
        $redmond=0;
      } else
      {
        open(MSBS,">$redmond"); # touch
        close(MSBS);
      }

      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o install -p 80 -t apache -m \"$siteName\" -w \"\~$user\" -u \"$user\" -g \"$site\" -pw \"$$\" -s \"$Htconf/httpd.conf\" -xu \"$user\" -xg \"$site\" >> /var/cobalt/sauce.log") && ($could_not_install = 1);

      # ignore anything that fpx did to this file
      unlink($redmond) if ($redmond);

      # Fpx use web access uses mod_auth_pam, which is tripped up by the default ~user/.htaccess
      unlink((getpwnam($user))[7]."/$Webdir/.htaccess");

      # FPX creates a bunch of .htaccess files that need to be re-written for
      # pam/shadow support.. So, find the new .htaccess files and edit each one
      # We do this before restoring original .htaccess files
      fpx_rewrite_htaccess((getpwnam($user))[7]."/$Webdir",'user',"$user $Adminuser");

      fpx_restore_htaccess((getpwnam($user))[7]."/$Webdir",@htax); # Fix what we've started
      return MSG_error("fpx_cant_install") if ($could_not_install);

      my($user_webbase) = $user_home.'/'.$Webdir;

      # Explicitly copy in binaries as FP web type apache-fp will disable this in admsrvexe
      foreach $fpbin ('/_vti_bin/fpcount.exe','/_vti_bin/shtml.exe','/_vti_bin/_vti_aut/author.exe')
      # For user subweb support add: ,'/_vti_bin/_vti_adm/admin.exe')
      {
        copy("/usr/local/frontpage/currentversion/exes$fpbin","$user_webbase$fpbin");
        chown($user_uid, $site_gid, "$user_webbase/$fpbin");
        chmod(04755,"$user_webbase/$fpbin");
      }

      # Update Apache conf explicitly
      my(@fpx_users)=Cobalt::Vsite::vsite_get_fpx($site);
      push(@fpx_users,$user); # Add user to site fpx users
      my($return)=Cobalt::Vsite::vsite_set_fpx($site,1,@fpx_users);
      return $return if ($return !~ /^2/);

      # Restore lost sticky bits
      fpx_recurse_chmod( 02775, "$user_home/$Webdir");

      return MSG_ok("fpx_ok_installed");
    } else # Disable
    {
      if ( -e "$Userdir/$site/$Webdir/_vti_pvt/services.cnf" )
      {
        open( SRVCS, "> $Userdir/$site/$Webdir/_vti_pvt/services.new" ) ||
          return MSG_error("fpx_cant_uninstall");
        open( ORIG, "$Userdir/$site/$Webdir/_vti_pvt/services.cnf" ) ||
          return MSG_error("fpx_cant_uninstall");
        # beware of periods...
        my $clean_user= $user;
        $clean_user=~ s/\./\\\./g;
 
        while(<ORIG>)
        {
          print SRVCS unless ( /^\/\~$clean_user\n*$/ );
        } 
        close( ORIG );
        close( SRVCS );

        move( "$Userdir/$site/$Webdir/_vti_pvt/services.new",
              "$Userdir/$site/$Webdir/_vti_pvt/services.cnf" ) ||
          return MSG_error("fpx_cant_uninstall");

        my $nobody_uid = (getpwnam('nobody'))[2];
        chown($nobody_uid, $site_gid, "$Userdir/$site/$Webdir/_vti_pvt/services.cnf") ||
          warn "could not change $Userdir/$site/$Webdir/_vti_pvt/services.cnf ownership: $!";
      }
      else
      {
        return MSG_error("fpx_cant_uninstall");
      }

      # Update Apache conf explicitly
      my(@old_users)=Cobalt::Vsite::vsite_get_fpx($site);
      my(@fpx_users,$fpx_user);
      foreach $fpx_user (@old_users)
      {
        push(@fpx_users,$fpx_user) unless ( ($fpx_user eq $user) ||
                                            ($fpx_user eq $site) );
      }
      my($return)=Cobalt::Vsite::vsite_set_fpx($site,1,@fpx_users);
      return $return if ($return !~ /^2/);

      # FPX files have been made unavailable to fpadmsrv.exe
      system("/bin/rm -rf $user_home/$Webdir/_vti* >> /var/cobalt/sauce.log 2>&1; /bin/rm -rf $user_home/$Webdir/_private > /dev/null 2>&1\n");

      return MSG_ok("services_fpx_ok_uninstalled");
    }

  } else                  ### Site web
  {
    # fpadmsrv.exe matches $siteName in httpd.conf
    my($siteName)=(Cobalt::Vsite::vsite_get_bygroup($group))[1];
    return MSG_error("fpx_invalid_badsite") unless ($siteName);

    if ($state) # Enable
    {
      # Rename .htaccess files so fpadmsrv.exe doesn't die from confusion
      my(@htax) = fpx_rename_htaccess("$Userdir/$group/$Webdir");

      my($user)='nobody';

      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o install -p 80 -t apache-fp -m \"$siteName\" -u \"webmaster\" -pw \"$wembasterPW\" -s \"$Htconf/httpd.conf\" -xu \"$user\" -xg \"$group\" >> /var/cobalt/sauce.log") && ($could_not_install = $!);

      # Fix what we've started for clean aborts
      fpx_restore_htaccess( "$Userdir/$group/$Webdir", @htax); 
      return MSG_error("fpx_cant_install") if ($could_not_install);

      # Update Apache access levels (pooch security)
      my($return)=Cobalt::Vsite::vsite_set_fpx($group,1);
      return $return if ($return !~ /^2/);

      # Add SMTP capability to FPX forms
      if (-w "$Fpx_cnfdir/$siteName:80.cnf")
      {
        my($orig_cnf);
	open(CNF);
        while(<CNF>) { $orig_cnf .= $_; }
        close(CNF);

	unless($orig_cnf =~ /smtphost/i)
	{
	  open(NUCNF,">> $Fpx_cnfdir/$siteName:80.cnf");
	  print NUCNF "SMTPHost:127.0.0.1\n";
	  print NUCNF "MailSender:nobody\@$siteName\n";
	  close(NUCNF);
	}

	# Must be readable by nobody
	chmod(0644,"$Fpx_cnfdir/$siteName:80.cnf");
      }

      # Restore lost sticky bits
      fpx_recurse_chmod( 02775, "$Userdir/$group/$Webdir");

      # Restore ShopSite ownership & permissions
      if (-d "/home/sites/$group/$Webdir/store/cgi-bin/ss")
      {
        system("/bin/chown -R $AdminUser.$group /home/sites/$group/$Webdir/store > /dev/null 2>&1");
        fpx_recurse_chmod( 02755, "$Userdir/$group/$Webdir/store");
      }

      return MSG_ok("fpx_ok_installed");

    } else # Disable
    {
      # Uninstall
      
      # disbale for users within the site
      fpx_disbale_userwebs($group);
      
      # Call fpadmsrv.exe (ugh)
      system("$Fpx_admsrvexe -o uninstall -p 80 -t apache-fp -m \"$siteName\" -s \"$Htconf/httpd.conf\" >> /var/cobalt/sauce.log") && return MSG_error("fpx_cant_uninstall");

      # Update Apache conf manually (restore security if possible)
      my($return)=Cobalt::Vsite::vsite_set_fpx($group,0);
      return $return if ($return !~ /^2/);

      # fpadmsrv should do this automatically 
      #  ..but we don't trusts its now does we..
      unlink("$Fpx_cnfdir/$siteName:80.cnf");

      # fpadmsrv.exe won't delete all stub cgis or old logs
      system("/bin/rm -rf $Userdir/$group/$Webdir/_vti* >> /var/cobalt/sauce.log 2>&1; /bin/rm -rf $Userdir/$group/$Webdir/_private >> /var/cobalt/sauce.log 2>&1\n");

      return MSG_ok("services_fpx_ok_uninstalled");
    }

    return MSG_error("fpx_cant_install") if ($could_not_install);
    return MSG_ok("fpx_ok_installed");
  }
}                                               

sub fpx_get_web
# Are FrontPage Server Extensions enabled?
# Returns 1 if FP web is present, 0 otherwise
# Arguments: user or site group name
# Side effects: none
{
  my($group)=@_;
  
  return 0 unless ($Features{'fpx'});
  return 0 unless (-x $Fpx_admsrvexe);

  # Test user vs group/site
  my($user);
  if(Cobalt::User::user_exist($group))  ### User web
  {
    my $home = (getpwnam($group))[7];
    return 1 if (-e "$home/$Webdir/_vti_pvt/service.cnf");
    return 0;
  } else
  {
    my($siteName)=(Cobalt::Vsite::vsite_get_bygroup($group))[1];
    # If the site root web exists it's going to have a config
    return 1 if (-e "$Fpx_cnfdir/$siteName:80.cnf");
  }
  return 0; 
}

sub fpx_rename_htaccess
# Rename .htaccess files to something else
# Arguments: directory root
# Return value: array of qualified files moved
{
  my($basedir)=@_;

  my(@htax,$hta);
  open(HTAS,"/usr/bin/find $basedir -name .htaccess -print |")
    || return();
  while(<HTAS>) { chop; push(@htax,$_); }
  foreach $hta (@htax) 
  {
    move("$hta","$hta.FP98") unless ($hta eq "$Htgroup_dir/$MainGroup/$Webdir/.htaccess");
  }
  close HTAS;
  return(@htax);
}

sub fpx_restore_htaccess
# Restores ftp_rename_htaccess action
# Arguments: FP Web directory, array of qualified files to move
# Return value: true or false
# Side affect: Removes .htaccess file from web root if FP added one
#   This seems to be a common bug, where a corrupt .htaccess appears
#   out of nowhere.
{
  my($webroot,@htax)=@_;

  my($hta);
  foreach $hta (@htax) 
  {
    move("$hta.FP98","$hta");
  }
  return 1;
}

sub fpx_rewrite_htaccess
# Rewrites selected fpx-generated .htaccess files so they'll reference 
# real unix users for root webs only.  Subwebs continue to use FP-only users.
#
# Arguments: directory root, literal group/user, group or username string
# Return value: array of edited files
{
  my($basedir,$flavour,$user)=@_;

  my(@htax,@edited,$hta,$newfile);
  open(HTAS,"/usr/bin/find $basedir -name .htaccess -print |")
    || return();
  while(<HTAS>) { chop; push(@htax,$_); }
  foreach $hta (@htax) 
  {
    next if ($hta eq "$basedir/.htaccess"); # root web exclusion

    open(HTA,$hta) || next;
    while(<HTA>)
    {
      s/^AuthUserFile.*$//g;
      s/^AuthGroupFile.*$//g;
      s/^require.*$/require $flavour $user\n/g;
      s/Real_User_Real_User/Real_User/g;
      $newfile.=$_;
    }
    close(HTA);

    # Clobber old file
    open(HTN,">$hta") || next;
    print HTN $newfile;
    close(HTN); 
 
    push(@edited,$hta);
    $newfile=''; # ..primitive
  }
  close HTAS;
  return(@edited);
}

sub fpx_recurse_chmod
# Resets web directory permissions post-FPX install
# Arguments: directory mode, root directory
{
  my( $file_mode, $root_dir)=@_;
  $file_mode ||= "02775"; # Defaults are good
  return unless ( -d $root_dir);

  chmod( $file_mode, $root_dir);

  opendir( SBDR, $root_dir) || return;
  map fpx_recurse_chmod($file_mode, "$root_dir/$_"),
    grep +(-d "$root_dir/$_" and !/^\.{1,2}$/), readdir(SBDR);
  close (SBDR);
}

sub fpx_disbale_userwebs
# Turns off all user webs for a site
# Arguments: group 
{
  my $group = shift;
  my @enabled;

  # Get list
  my @fpx_users = Cobalt::Vsite::vsite_get_users($group);
  for (@fpx_users) { ( fpx_get_web($_) ) ? push @enabled, $_ : 0; }

  # Disable the Enabled
  foreach my $user (@enabled) { 
    my $uobj=Cobalt::Meta->create("type" => "users");
    $uobj->retrieve($user);
    fpx_set_web($user,0);
    $uobj->put("fpx"=>'off');
    $uobj->save;
  }

  return;
}






