## Cobalt::Meta::vsite
##

package Cobalt::Meta::vsite;

BEGIN {

    use Cobalt::Meta ();

    use vars   qw[ $VERSION @ISA ];

    @ISA     = qw[ Cobalt::Meta ];
    $VERSION = 1.01;

    require Cobalt::Util;
    import Cobalt::Util qw[ &MSG_get_ui ];

    require Cobalt::BandWidth;
    require Cobalt::Netutil;
    require Cobalt::Admserv;
    require Cobalt::Layout;
    require Cobalt::Group;
    require Cobalt::Vsite;
    require Cobalt::Email;
    require Cobalt::Http;
    require Cobalt::List;
    require Cobalt::Ftp;
    require Cobalt::Fpx;

    require Cobalt::Product;
    import Cobalt::Product qw[ $MainGroup $AdminPages_dir $Htgroup_dir
			       $Vol_home $Vol_rootdir $Webdir $Sites_DelLock 
	               $Cron_basedir ];

}

use vars qw[ @webSvrAli @emailSvrAli $fpxPassword ];
@emailSvrAli = ();
@webSvrAli   = ();
$fpxPassword = '';

###### Cobalt::Meta::vsite->new()
#
# creates a new virtual site Meta object
#
sub new
{
    my $proto = shift;
    my %param = @_;
    my $class = ref($proto) || $proto;
    my %init = (
	type           => 'vsite',
	name           => undef,
	ipaddr         => undef,
	hostname       => undef,
	domain         => undef,
	fqdn           => undef,
        volume         => undef,
	quota          => 200,
	ftpquota       => 20,
	ftpusers       => 10,
	maxusers       => 25,
	ftp            => 'f',
	cgi            => 'f',
	ssi            => 'f',
	ssl            => 'f',
	fpx            => 'f',
	apop           => 'f',
	casp           => 'f',
	php            => 'f',
	shell          => 'f',
	suspend        => 'f',
	user_fpx       => 'f',
	user_casp      => 'f',
	user_shell     => 'f',
	user_apop      => 'f',
	user_namegen   => 0,
	user_quota     => 20,
	userlist_range => 15,
	userlist_sort  => 'name',
    );

    my $self = new Cobalt::Meta(%init);
    if (%param) { $self->put(%param) }

    bless($self, $class);
    return $self;
}

sub DESTROY
{
    my $self = shift;
    @emailSvrAli = ();
    @webSvrAli   = ();
    $fpxPassword = '';
    return;
}

sub version {
    return $VERSION;
}

sub name {
    my $self = shift;
    if (@_) { $self->{name} = shift }
    return $self->{name};
}

sub ipaddr {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (Cobalt::Netutil::netutil_test_ipaddr($_)) {
	    $self->{ipaddr} = $_;
	}
    }
    return $self->{ipaddr};
}

sub hostname {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (/^[a-zA-Z0-9\-\.]*$/) { $self->{hostname} = $_ }
    }
    return $self->{hostname};
}

sub domain {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (/^[a-zA-Z0-9\-\.]*$/) { $self->{domain} = $_ }
    }
    return $self->{domain};
}

sub fqdn {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (/^[a-zA-Z0-9\-\.]*$/) { $self->{fqdn} = $_ }
    }
    return $self->{fqdn};
}

sub quota {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (/^\d+$/o && $_ > 0) { $self->{quota} = $_ }
	elsif (/^0?$/) { $self->{quota} = 0 }
    }
    return $self->{quota};
}

sub ftpquota {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (/^\d+$/o && $_ < $self->{quota}) { $self->{ftpquota} = $_ }
	elsif (/^0?$/) { $self->{ftpquota} = 0 }
    }
    return $self->{ftpquota};
}

sub ftpusers {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (/^\d+$/o && $_ > 0) { $self->{ftpusers} = $_ }
	elsif (/^0?$/) { $self->{ftpusers} = 0 }
    }
    return $self->{ftpusers};
}

sub maxusers {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if (/^\d+$/o && $_ > 0) { $self->{maxusers} = $_ }
	elsif (/^0?$/) { $self->{maxusers} = 0 }
    }
    return $self->{maxusers};
}

sub volume {
    my $self = shift;
    if (@_) {
	$_ = shift;
	$self->{volume} = $_;
    }
    return $self->{volume};
}

sub ftp {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{ftp} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{ftp} = 'f' }
    }
    if ($self->{ftp} eq 't') { return 1 }
    return;
}

sub cgi {

    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{cgi} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{cgi} = 'f' }
    }
    if ($self->{cgi} eq 't') { return 1 }
    return;
}

sub ssi {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{ssi} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{ssi} = 'f' }
    }
    if ($self->{ssi} eq 't') { return 1 }
    return;
}

sub ssl {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{ssl} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{ssl} = 'f' }
    }
    if ($self->{ssl} eq 't') { return 1 }
    return;
}

sub fpx {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{fpx} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{fpx} = 'f' }
    }
    if ($self->{fpx} eq 't') { return 1 }
    return;
}

sub apop {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{apop} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{apop} = 'f' }
    }
    if ($self->{apop} eq 't') { return 1 }
    return;
}

sub casp {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{casp} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{casp} = 'f' }
    }
    if ($self->{casp} eq 't') { return 1 }
    return;
}

sub php {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{php} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{php} = 'f' }
    }
    if ($self->{php} eq 't') { return 1 }
    return;
}

sub shell {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{shell} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{shell} = 'f' }
    }
    if ($self->{shell} eq 't') { return 1 }
    return;
}

sub suspend {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{suspend} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{suspend} = 'f' }
    }
    if ($self->{suspend} eq 't') { return 1 }
    return;
}

sub user_fpx {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{user_fpx} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{user_fpx} = 'f' }
    }
    if ($self->{user_fpx} eq 't') { return 1 }
    return;
}

sub user_casp {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{user_casp} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{user_casp} = 'f' }
    }
    if ($self->{user_casp} eq 't') { return 1 }
    return;
}

sub user_shell {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{user_shell} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{user_shell} = 'f' }
    }
    if ($self->{user_shell} eq 't') { return 1 }
    return;
}

sub user_apop {
    my $self = shift;
    if (@_) {
	$_ = shift;
	if    (/^(t|true|yes|on|1)$/oi)   { $self->{user_apop} = 't' }
	elsif (/^(f|false|no|off|0)?$/oi) { $self->{user_apop} = 'f' }
    }
    if ($self->{user_apop} eq 't') { return 1 }
    return;
}

sub user_quota {
    my $self = shift;
    if (@_) { 
	$_ = shift;
	if (/^\d+$/o && $_ > 0) { $self->{user_quota} = $_ }
	elsif (/^0?$/) { $self->{user_quota} = 0 }
    }
    return $self->{user_quota};
}

sub user_namegen {
    my $self = shift;
    if (@_) { $self->{user_namegen} = shift }
    return $self->{user_namegen};
}

sub userlist_range {
    my $self = shift;
    if (@_) { $self->{userlist_range} = shift }
    return $self->{userlist_range};
}

sub userlist_sort {
    my $self = shift;
    if (@_) { $self->{userlist_sort} = shift }
    return $self->{userlist_sort};
}

sub webServerAlias {
    my $self = shift;
    foreach my $a (@_) {
	next if ($a =~ /^[\s,]*$/);
	next if grep /^$a$/, @webSvrAli;
	push @webSvrAli, $a;
    }
    return @webSvrAli;
}

sub emailServerAlias {
    my $self = shift;
    foreach my $a (@_) {
	next if ($a =~ /^[\s,]*$/);
	next if grep /^$a$/, @emailSvrAli;
	push @emailSvrAli, $a;
    }
    return @emailSvrAli;
}

sub fpxPassword {
    my $self = shift;
    if (@_) {
        $fpxPassword = shift;
    }
    return $fpxPassword;
}


###### Cobalt::Meta::vsite->verify()
#
# inputs: Meta vsite object
# return: @info - ARRAY of localized informational error message
#         zero length ARRAY indicates success
#             [ !scalar(@info) ]
#
sub verify
{
    my $self = shift;
    my @info = ();

    return ("invalid") unless ($self && $self->{type} eq 'vsite');

      # generate a group name
    $self->{name} = Cobalt::Vsite::vsite_gen_name($self->{fqdn});
    unless ($self->{name}) {
	push @info, MSG_get_ui("vsite_cant_gen_name");
    }

      # trim leading dots
    $self->{domain} =~ s/^\.//g;

      # generate Fully Qualified Domain Name (fqdn)
    $self->fqdn(qq[$self->{hostname}.$self->{domain}]);

      # check for essential data
    unless ($self->{ipaddr} && $self->{quota}) {
	push @info, MSG_get_ui("vsite_essential_data");
    }
    unless ($self->{hostname} && $self->{domain} && $self->{fqdn}) {
        push @info, MSG_get_ui("hostNameRequired");
    }

      # check if volume mountpoint exists
    if (! $self->{volume}) {
	$self->{volume} = $Vol_home;
    }
    elsif ($self->{volume} ne $Vol_home && ! -d qq[$Vol_rootdir/$self->{volume}]) {
	push @info, MSG_get_ui("volNameRequired");
    }

      # check syntax of host,domain,fqdn
    if ($self->{hostname} !~ /^[a-zA-Z0-9\-\.]*$/) {
	push @info, MSG_get_ui("hostNameNotValid", $self->{hostname});
    }
    if ($self->{domain} !~ /^[a-zA-Z0-9\-\.]*$/) {
	push @info, MSG_get_ui("domainNameNotValid", $self->{domain});
    }
    if ($self->{fqdn} =~ /localhost/io) {
	push @info, MSG_get_ui("localhostNotAllowed");
    }
    if ($self->{fqdn} !~ /^[a-zA-Z0-9\-\.]*$/) {
	push @info, MSG_get_ui("hostNameNotValid1");
    }

      # find out if the same fqdn is used by any other vsite
    if (scalar($self->SQL(qq[ SELECT name FROM vsite             ],
			  qq[ WHERE name <> 'default'            ],
			  qq[ AND hostname = '$self->{hostname}' ],
			  qq[ AND domain = '$self->{domain}'     ]))) {
	push @info, MSG_get_ui("vsite_cant_hostnameinuse", $self->{fqdn});
    }

      # verify the IP address
    unless (Cobalt::Netutil::netutil_test_ipaddr($self->{ipaddr})) {
	push @info, MSG_get_ui("addrNotValid");
    }

      # test web server alias for domain validity
    foreach ($self->webServerAlias) {
	unless (/^[a-zA-Z0-9\-\.]*$/) {
	    push @info, MSG_get_ui("webServerAliasInvalid", $_);
	}
    }

      # test email server alias for domain validity
    foreach ($self->emailServerAlias) {
	unless (/^[a-zA-Z0-9\-\.]*$/) {
	    push @info, MSG_get_ui("emailServerAliasInvalid", $_);
	}
    }

      # check disk quota and frontpage minimum
    if ($self->{quota} !~ /^\d+$/o || $self->{quota} <= 0) {
	push @info, MSG_get_ui("diskQuotaNotPositive");
    }

      # check SSL settings
    if ($self->ssl) {
	if (scalar($self->SQL(qq[ SELECT name FROM vsite         ],
			      qq[ WHERE name <> 'default'        ],
			      qq[ AND ipaddr = '$self->{ipaddr}' ],
			      qq[ AND ssl IS TRUE                ]))) {
	    push @info, MSG_get_ui("vsite_cant_ipinuse_ssl");
	}
    }

      # check anonymous FTP settings
    if ($self->ftp) {
	if (scalar($self->SQL(qq[ SELECT name FROM vsite         ],
			      qq[ WHERE name <> 'default'        ],
			      qq[ AND ipaddr = '$self->{ipaddr}' ],
			      qq[ AND ftp IS TRUE                ]))) {
	    push @info, MSG_get_ui("vsite_cant_ipinuse_ftp");
	}
	if ($self->{ftpquota} !~ /^\d+$/o) {
	    push @info, MSG_get_ui("ftpQuotaNotInteger");
	}
	if ($self->{ftpquota} > $self->{quota}) {
	    push @info, MSG_get_ui("ftpQuotaTooBig", $self->{quota});
	}
	if ($self->{ftpusers} !~ /^\d+$/o) {
	    push @info, MSG_get_ui("ftpUsersNotInteger");
	}
	if ($self->{ftpusers} == 0) {
	    push @info, MSG_get_ui("ftpUsersNotPositive");
	}
    }

    return @info;
}

###### Cobalt::Meta::vsite->add()
#
# inputs: Meta vsite object
# return: ARRAY($success, \@info)
#         !$success means FATAL errors
#            [ stored in ARRAY ref \@info ]
#         $success means no FATAL errors, possibility of WARNING(s)
#            [ stored in ARRAY ref \@info     ]
#            [ scalar(@$info) = # of warnings ]
#
sub add
{
    my $self = shift;
    my @info = ();
    my $ret  = '';

    return (0, ['invalid']) unless ($self && $self->{type} eq 'vsite');

      # verify the vsite object
    my @errorlog = $self->verify;
    return (0, \@errorlog) if (scalar(@errorlog));

      # make sure we generated a valid group name
    unless ($self->{name} =~ /^[\w\-\.]+$/o) {
	$ret = Cobalt::Util::MSG_get('4234', $self->{name});
	return (0, [$ret]);
    }

      # determine home directory mountpoint
    my $mount = '';
    if ($self->{volume} && $self->{volume} ne $Vol_home) {
	$mount = qq[$Vol_rootdir/$self->{volume}];
    }
      # try to add group
    $ret = Cobalt::Group::group_add($self->{name}, int(1024 * $self->{quota}), $mount);
    unless ($ret =~ /^2/) {
	return (0, [ +substr($ret, 4) ]);
    }

      # try to do vsite addition
    $ret = Cobalt::Vsite::vsite_add($self->{ipaddr}, $self->{fqdn}, $self->{name});
    unless ($ret =~ /^2/) {
	Cobalt::Group::group_del($self->{name});
	return (0, [ +substr($ret, 4) ]);
    }

      # setup site administration pages
    $ret = Cobalt::Layout::layout_siteManage($self->{name}, $self->{fqdn});
    $ret = Cobalt::Admserv::admserv_set_access(qq[$AdminPages_dir/siteManage/$self->{name}],
					       'require', 'group', $self->{name});
    unless ($ret =~ /^2/) {
	push @info, substr($ret, 4);
    }

      # substitute variables in index.html
    my $webindex = qq[$Htgroup_dir/$self->{name}/$Webdir/index.html];
    if (-e $webindex) {
	if (Cobalt::Util::FILE_lock_rw($webindex, *IN, *OUT)) {
	    while (<IN>) {
		s/\[DOMAIN\]/$$self{fqdn}/go;
		print OUT;
	    }
	    Cobalt::Util::FILE_unlock_rw($webindex, *IN, *OUT);
	}

	  # restore index permissions
	chown((getpwnam('nobody'))[2], (getgrnam($self->{name}))[2], $webindex);
	chmod(0664,$webindex);
    }

      # Setup a virtual list server.
    $ret = Cobalt::List::list_create_virtual($self->{name}, $self->{fqdn});
    unless ($ret =~ /^2/) {
	push @info, substr($ret, 4);
    }
	
	$ret = Cobalt::Email::mail_virtuser_dfl_aliases($self->{fqdn},1);
    unless ($ret =~ /^2/) {
	push @info, substr($ret, 4);
    }


      # setup logrotate entries (use quota/10 MB)
    $ret = Cobalt::Vsite::vsite_set_logrotate($self->{name}, (int($self->{quota} / 10) or 1));
    unless ($ret =~ /^2/) {
	push @info, substr($ret, 4);
    }

      # tell sendmail to accept mail at this fqdn
    $ret = Cobalt::Email::mail_add_cw($self->{fqdn});
    unless ($ret =~ /^2/) {
	push @info, substr($ret, 4);
    }

      # add IP address to relay list
    $ret = Cobalt::Email::mail_add_relay($self->{ipaddr});
    unless ($ret =~ /^2/) {
	push @info, substr($ret, 4);
    }

      # handle web server aliases
    if (scalar(@webSvrAli)) {

	my @goodal;
	my %webal = map { $_ => 1; } (Cobalt::Vsite::vsite_list_alias());

	  # compare against ALL web server aliases
	foreach (@webSvrAli) {
	    if (exists($webal{$_})) {
		push @info, MSG_get_ui("webServerAliasInUse", $_);
		next;
	    }
	    push @goodal, $_;
	}

	  # now add those that passed the tests
	if (scalar(@goodal)) {
	    $ret = Cobalt::Vsite::vsite_set_alias($self->{name}, @goodal);
	    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
	}
    }

      # handle email server aliases
    if (scalar(@emailSvrAli)) {

	my @goodal;
	my %emailal = Cobalt::Email::mail_virtuser_list_route();

	foreach (@emailSvrAli) {
	    if (exists($emailal{$_})) {
		push @info, MSG_get_ui("emailServerAliasInUse", $_);
		next;
	    }
	    # Cannot allow $fqdn as an @esAlias entry 
	    # because virtusertable will not rebuild.
	    # Using different warning...
	    if ($self->{fqdn} eq $_)  {
		push @info, MSG_get_ui("emailServerAliasInvalid", $_);
		next;
	    }
	    push @goodal, $_;
	}
	  # tell sendmail that this address is local
	$ret = Cobalt::Email::mail_add_cw(@goodal);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

	  # route email @server.alias.com --> @host.domain.com
	%emailal = map { $_ => $self->{fqdn} } @goodal;
	$ret = Cobalt::Email::mail_virtuser_add_route(%emailal);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

    }

      # configure FTP (and anonymous ftp if enabled)
    if ($self->ftp) {
	$ret = Cobalt::Ftp::ftp_config($self->{ipaddr}, 1, 1, $self->{name},
				       $self->{ftpusers}, $self->{ftpquota});
    } else {
	$ret = Cobalt::Ftp::ftp_config($self->{ipaddr}, 1, 0, $self->{name});
    }
    unless ($ret =~ /^2/) {
	$self->ftp('FALSE');
	push @info, substr($ret, 4);
    }

      # configure CGI scripts
    if ($self->cgi) {
	$ret = Cobalt::Vsite::vsite_set_cgis($self->{name}, 1);
	unless ($ret =~ /^2/) {
	    $self->cgi('FALSE');
	    push @info, substr($ret, 4);
	}
    }

      # configure SSI
    if ($self->ssi) {
	$ret = Cobalt::Vsite::vsite_set_ssi($self->{name}, 1);
	unless ($ret =~ /^2/) {
	    $self->ssi('FALSE');
	    push @info, substr($ret, 4);
	}
    }

      # configure PHP
    if ($self->php) {
	$ret = Cobalt::Vsite::vsite_set_php($self->{name}, 1);
	unless ($ret =~ /^2/) {
	    $self->php('FALSE');
	    push @info, substr($ret, 4);
	}
    }

      # configure SSL (note: does not return useful status)
    if ($self->ssl) {
        $ret = Cobalt::Vsite::vsite_set_sslconf($self->{name}, 1);
    }

      # configure Frontpage extensions
    if ($self->fpx) {
	$ret = Cobalt::Fpx::fpx_set_web($self->{name}, 1, $fpxPassword);
	unless ($ret =~ /^2/) {
	    $self->fpx('FALSE');
	    push @info, substr($ret, 4);
	}
    }

      # configure Chilisoft ASP
    if ($self->casp) {
	$ret = Cobalt::Vsite::vsite_set_casp($self->{name}, 1);
	unless ($ret =~ /^2/) {
	    $self->casp('FALSE');
	    push @info, substr($ret, 4);
	}
    }

      # restart daemons
    Cobalt::Http::http_restart();
    Cobalt::Email::mail_sendmail_hup();

      # save to database
    $self->save();

    return (1, \@info);
}


###### Cobalt::Meta::vsite->del()
#
# inputs: Meta vsite object
# return: ARRAY($success, \@info)
#         !$success means FATAL errors
#            [ stored in ARRAY ref \@info ]
#         $success means no FATAL errors, possibility of WARNING(s)
#            [ stored in ARRAY ref \@info     ]
#            [ scalar(@$info) = # of warnings ]
#
sub del
{
    my $self = shift;
    my @info = ();
    my $ret  = '';

    return (0, ['invalid']) unless ($self && $self->{type} eq 'vsite');

    if ($self->{name} eq $MainGroup) {
	return (0, ['The primary site cannot be deleted.']);
    }

      # check for existing lock
    if (-e $Sites_DelLock) {
	sleep(3);
	if (-e $Sites_DelLock) {
	    $ret = MSG_get_ui("vsite_cant_deleting_other_vsite");
	    return (0, [$ret]);
	}
    }
      # try lame locking
    if (Cobalt::Util::FILE_lock_w($Sites_DelLock, *LCK)) {
	print LCK $$;
        Cobalt::Util::FILE_unlock(*LCK);
    } else {
	$ret = MSG_get_ui("vsite_cant_deleting_other_vsite");
	return (0, [$ret]);
    }

      # delete all mailing lists belonging to this site
    my @lists = Cobalt::List::list_lists($self->{name}, $self->{fqdn});
    if (scalar(@lists)) {
	if ($lists[0] =~ /^[134]\S*\s/) {
	    push @info, substr($lists[0], 4);
	} 
	$ret = Cobalt::List::list_delete($self->{name}, $self->{fqdn}, @lists);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
    }

      # delete the virtual list server
    $ret = Cobalt::List::list_delete_virtual($self->{name}, $self->{fqdn});
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

      # delete all site users
    foreach (Cobalt::Vsite::vsite_get_users($self->{name})) {
	next if (/^admin$/o);
	$ret = Cobalt::User::user_del($_);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
    }
    $self->SQL(qq[ DELETE FROM users             ],
	       qq[ WHERE vsite = '$self->{name}' ]);

	$ret = Cobalt::Email::mail_virtuser_dfl_aliases($self->{fqdn},0);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

      # disable anonymous FTP iff it was enabled
    if (Cobalt::Ftp::ftp_get_anonymous($self->{name})) {
	$ret = Cobalt::Ftp::ftp_config($self->{ipaddr}, 1, 0, $self->{name});
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
    }

      # disable casp (necessary to down daemons if no other sites need them)
    if ($self->get("casp") eq 'on') {
        $ret = Cobalt::Vsite::vsite_set_casp($self->{name}, 0);
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
    }

	  # disable frontpage if enabled
	  # this cleans up /usr/local/frontpage/fqdn:80.cnf
	if($self->get("fpx") eq 'on') {
		$ret = Cobalt::Fpx::fpx_set_web($self->{name}, 0);
    	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
	}

      # delete virtual host entry from Apache conf
    $ret = Cobalt::Vsite::vsite_del($self->{name});
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

      # disable logfile rotation
    $ret = Cobalt::Vsite::vsite_set_logrotate($self->{name}, 0);
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

      # delete email server aliases
    my %al = Cobalt::Email::mail_virtuser_list_route($self->{fqdn});

      # route email @server.alias.com to @host.domain.com 
    $ret = Cobalt::Email::mail_virtuser_del_route(%al);
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }
    
      # tell sendmail not to accept mail at aliases
    $ret = Cobalt::Email::mail_del_cw(keys(%al));
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

      # remove fqdn from local-host-names
    $ret = Cobalt::Email::mail_del_cw($self->{fqdn});
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

		# revmove the scheduled backup
	my @crons = qw( daily weekly monthly );
	for (@crons) {
		my $schFile = $Cron_basedir.$_."/scheduled-backup.".$self->{name};
		unlink $schFile if (-e $schFile);
	}

      # things to do if this is the last vsite using this IP
    unless (scalar($self->SQL(qq[ SELECT name FROM vsite],
			      qq[ WHERE ipaddr = '$self->{ipaddr}' ],
			      qq[ AND name != 'default' ],
			      qq[ AND name != '$self->{name}' ])))
    {
	  # remove IP from relay list
	$ret = Cobalt::Email::mail_del_relay($self->{ipaddr});
	unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

	  # delete bandwidth limit
	Cobalt::BandWidth::bw_delip($self->{ipaddr});
    }

      # disable ssl (note: does not return useful status)
    if ($self->ssl) {
	$ret = Cobalt::Vsite::vsite_set_sslconf($self->{name}, 0);
    }

      # delete site management web pages
    qx[ /bin/rm -fr $AdminPages_dir/siteManage/$self->{name} ];

      # delete system group
    $ret = Cobalt::Group::group_del($self->{name});
    unless ($ret =~ /^2/) { push @info, substr($ret, 4) }

      # restart daemons
    Cobalt::Email::mail_sendmail_hup();
    Cobalt::Http::http_restart();

      # delete database entry
    $self->delete();

    unlink $Sites_DelLock if (-e $Sites_DelLock); 
    
    return (1, \@info);
}

1;

__END__
