# Vsite.pm: Functions for virtual hosting
#
# $Id: Vsite.pm,v 1.123.2.1.2.3 2001/07/19 22:29:50 dtiu Exp $
#
# Written by
#   Will DeHaan (will@cobaltnet.com)
#   Brion Moss (brion@cobaltnet.com, brion@nda.com)
#   Duncan Laurie <duncan@cobaltnet.com>
#   Harris Vaegan-Lloyd <harris@cobaltnet.com>
#
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Vsite;

BEGIN {

    require Exporter;
    require SelfLoader;

    use vars qw(@ISA @EXPORT @EXPORT_OK);

    @ISA    = qw[ Exporter SelfLoader ];
    @EXPORT = qw[ &vsite_add &vsite_del &vsite_gen_name
		  &vsite_list &vsite_get_bygroup &vsite_getgroup &vsite_get_users

		  &vsite_disable &vsite_enable &vsite_issuspend

		  &vsite_admin_add &vsite_admin_del &vsite_admin_set &vsite_admin_list
		  &vsite_add_alias &vsite_del_alias &vsite_list_alias &vsite_set_alias

		  &vsite_set_cgis &vsite_get_cgis
		  &vsite_set_ssi &vsite_get_ssi
		  &vsite_set_casp &vsite_get_casp &vsite_get_casp_apps &vsite_set_casp_apps
		  &vsite_set_fpx &vsite_get_fpx
		  &vsite_set_php &vsite_get_php

		  &vsite_sync &vsite_migrate_ipaddr
		  &vsite_set_logrotate

		  &vsite_ssl_get_cert_info &vsite_check_cert
		  &vsite_ssl_set_identity &vsite_ssl_clear
		  &vsite_ssl_get_cert &vsite_ssl_set_cert
		  &vsite_get_ssl &vsite_set_sslconf

		  &vsite_get_xml
		];
}

use Cobalt::Product;
require Cobalt::Meta;
require Cobalt::Util;
import Cobalt::Util qw( MSG_ok MSG_error MSG_get );
use Cobalt::Group;
use Cobalt::List;
use Cobalt::Netutil;
use Cobalt::Network;
use Cobalt::Ftp;
use Cobalt::Fpx;
use Cobalt::Http;
use Cobalt::Ssl;
use Cobalt::Email;
use Cobalt::User;
use Cobalt::Layout;
use Cobalt::Admserv;

use DB_File;
use Fcntl qw(O_RDWR O_CREAT);
use integer;
require File::Copy;
import File::Copy qw( copy );

1;

__DATA__

sub vsite_add
# Adds a virtual web server
# Arguments: the IP address, server name, associated group name, 
#            admin email address, optional subnet mask, 
# Return value: string of the form MSG_error("3310")
# Side effects: Modifies httpd.conf, may add a virtual network interface
{
    my($addr,$name,$group,$serveradmin,$mask) = @_;
    my($ret,$found_interface,$found_namev,$conf,$new_conf,$found_group);

    # This is a new site, enforce a maximum site count
    #   "Not with RaQ 2" /wdh  
    # my (@siteslist) = Cobalt::Vsite::vsite_list();
    # return MSG_error("4597")
    #   if (($#siteslist + 1) >= $Sites_Max); # +1 to un-zero-index

    $serveradmin ||= $Adminuser; # Give it a default..
    # hack to prevent perl from joining the $ at the end of a here
    # document line to the text on the next line and do variable
    # interpolation on it...perl _is_ brain dead!!
    my $my = '$';		
    $new_conf .= <<EOF;
<VirtualHost $addr>
ServerName $name
ServerAdmin $serveradmin
DocumentRoot $Htgroup_dir/$group/$Webdir
RewriteEngine on
RewriteCond %{HTTP_HOST}                !^$addr(:80)?$my
RewriteCond %{HTTP_HOST}                !^$name(:80)?$my
RewriteRule ^/(.*)                      http://$name/\$1 [L,R]
RewriteOptions inherit
AliasMatch ^/~([^/]+)(/(.*))? $Htgroup_dir/$group/users/\$1/web/\$3
</VirtualHost>
EOF

    # 'update the current configuration
    open(HTCNF,"$Htconf/httpd.conf") ||
      return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (LOCK, ">$Lockdir/httpd.conf") ||
      return vsite_abort(MSG_error("vsite_wicked"));

    $found_interface = 0;
    while(<HTCNF>)
    {
	if (!$found_interface and
	    /^<VirtualHost $addr>$/ ... /^<\/VirtualHost>$/) {
	    # sanity check
	    return vsite_abort(MSG_error("vsite_cant_hostnameinuse", $name))
		if (/^DocumentRoot\s+$Userdir\/$group\/$Webdir$/ or
		    /^ServerName\s+$name$/);

	    # slurp!
	    $conf .= $_;

	    # end of section; throw up the slurped goo..
	    if (/^<\/VirtualHost>$/) {
		# first entry for $addr
		$found_interface = 1;
		print LOCK "\nNameVirtualHost $addr\n";

		# hack to get the maingroup conf first
		if ($group eq $MainGroup) {
		    print LOCK "${new_conf}${conf}";
		} else {
		    print LOCK "${conf}${new_conf}";
		}
		$conf = '';
	    }
	} else {
	    # spit it unless..
	    print LOCK unless (/NameVirtualHost $addr$/);
	}
    }
    print LOCK "\nNameVirtualHost $addr\n$new_conf" unless ($found_interface);
    close(HTCNF);
    close(LOCK);

    # Create web subdir
    unless(-d "$Htgroup_dir/$group/$Webdir")
    {
	mkdir("$Htgroup_dir/$group/$Webdir",775);
	chown((getpwnam('nobody'))[2],(getgrnam($group))[2],
	      "$Htgroup_dir/$group/$Webdir");
	chmod(02775,"$Htgroup_dir/$group/$Webdir");
    }

    # Create logs subdir
    unless(-d "$Htgroup_dir/$group/$Logdir")
    {
	mkdir("$Htgroup_dir/$group/$Logdir",775);
	chown((getpwnam('nobody'))[2],(getgrnam($group))[2],
	      "$Htgroup_dir/$group/$Logdir");
	chmod(02750,"$Htgroup_dir/$group/$Logdir");
    }

    # Create certs subdir
    unless(-d "$Htgroup_dir/$group/$Certdir")
    {
	mkdir("$Htgroup_dir/$group/$Certdir",775);
	chown((getpwnam('nobody'))[2],(getgrnam($group))[2],
	      "$Htgroup_dir/$group/$Certdir");
	chmod(02750,"$Htgroup_dir/$group/$Certdir");
    }

    # Add the network interface if necessary
    unless($group eq $MainGroup) # Special case handled by Network.pm
    {
	my(@all_IPs) = Cobalt::Network::net_get_alladdrs();
	my($ip_instance,%active_IPs);

	%active_IPs = map { $_ => 1; } @all_IPs;

	unless ($active_IPs{$addr})
	{
	    warn "Vsite::vsite_add found IP address $addr not to be in use. Adding interface..\n" if ($Debug);
	    $mask ||= (Cobalt::Network::net_get_ipaddr())[1];

	    # Insert subnet mask validation here..
	    ($ret)=net_set_vinterface($addr,$mask);
	    return vsite_abort($ret) unless ($ret =~ /^2/);

	    if ($Features{'proftpd'}) {
		warn "Vsite::vsite_add configuring ftp server for new ip address $addr\n" if ($Debug);
		($ret)=Cobalt::Ftp::ftp_set_vinterface($addr);
		return vsite_abort($ret) unless ($ret =~ /^2/);
	    }
	}
    }

    # Add mod_rewrite friendly symlink into the siteManage ui
    unlink("$AdminPages_dir/siteManage/$name") if (-l "$AdminPages_dir/siteManage/$name");
    symlink("$AdminPages_dir/siteManage/$group",
	    "$AdminPages_dir/siteManage/$name");
	
    # Add ip/siteadmin redirect
    unlink("$AdminPages_dir/siteManage/$addr")
        if (-l "$AdminPages_dir/siteManage/$addr");
    symlink("$AdminPages_dir/siteManage/$group",
            "$AdminPages_dir/siteManage/$addr"); 

    # Add filesystem friendly symlink into the data partition
    #   necessary with ambiguous site group names
    unlink("$Userdir/$name") if (-l "$Userdir/$name");
    symlink("$Htgroup_dir/$group","$Userdir/$name");

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("2660"); 
}

sub vsite_del
# Removes a name-based virtual web server
# Arguments: The site group name
# Return value: string of the form MSG_error("3311")
# Side effects: Modifies httpd.conf, may delete a virtual network interface
{
    my ($group) = @_;
    my($foundit,$found_web,$conf,$num_interface,$ret);
    my ($addr, $name);
    my @vsites = vsite_list();
    grep { ($addr, $name) = @$_ if ($group eq $$_[2]) } @vsites;
    $num_interface = scalar (grep { $addr eq $$_[0] } @vsites);
    return vsite_abort(MSG_error("vsite_cant_webnotfound", $name)) unless ($num_interface);

    # update the current configuration
    open(HTCNF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (LOCK, ">$Lockdir/httpd.conf") ||
	return vsite_abort(MSG_error("vsite_wicked"));

    $found_web = 0;
    while(<HTCNF>) 
    {
	if (/^<VirtualHost $addr>$/ ... /^<\/VirtualHost>$/)
	{
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$group\/$Webdir$/);
	    # slurp!
	    $conf .= $_;

	    if (/^<\/VirtualHost>$/)
	    {
		# don't spit if we got a match
		if ($found_web) {
		    $foundit=1;
		} else {
		    print LOCK $conf;
		}
		$conf = '';
		$found_web = 0;
	    }
	} elsif (/^NameVirtualHost $addr$/) {
	    print LOCK if ($num_interface > 1);
	} else {
	    # spit it..
	    print LOCK;
	}
    }
    close(HTCNF);
    close(LOCK);

    # remove the network interface if necessary
    unless($group eq $MainGroup) # Special case handled by Network.pm
    {
	if ($num_interface == 1)
	{
	    warn "Vsite::vsite_del removing virtual network support for the ip address $addr\n" if ($Debug);
	    net_del_vinterface($addr);
	    if ($Features{'proftpd'})
	    {
		warn "Vsite::vsite_del removing ftp server support for ip address $addr\n" if ($Debug);
		Cobalt::Ftp::ftp_del_vinterface($addr);
	    }
	}
    }

    # Remove mod_rewrite friendly symlink from the siteManage ui
    unlink("$AdminPages_dir/siteManage/$name")
	if (-l "$AdminPages_dir/siteManage/$name");

    # Remove filesystem friendly symlink from the data partition
    #   necessary with ambiguous site group names
    unlink("$Userdir/$name") if (-l "$Userdir/$name");

    Cobalt::Vsite::vsite_set_sslconf($group, 0);
    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_webdeleted"); 
}

sub vsite_gen_name
# Generate a back-end name that is associated with a virtual site
# Arguments: none
# Return value: an indexed group name (site0, site1, ...)
# Example: $siteGroup = vsite_gen_name();
{
    my(@allGroups)=Cobalt::Group::group_list();
    my(%activeGroups,$groupInst);
    foreach $groupInst (@allGroups) { $activeGroups{$groupInst}=1; }

    my($group,$nameIndex)=('site',1);
    while($activeGroups{$group.$nameIndex}) { $nameIndex++; }
    
    return($group.$nameIndex);
}

sub vsite_get_bygroup
# Get the information about a virtual interface by group name
# A wrapper to vsite_list, which is why $group is a parameter and a return value
# Arguments: group name
# Return value: array of (ip,hostname,group,webmaster email,subnet) or 0 
# Side effects: calls Cobalt::Vsite::vsite_list() to read the httpd.conf file
# Example: ($ip,$hostname,$group,$webmaster_email) = Cobalt::Vsite::vsite_get_bygroup("ratdogs");
{
    my ($group) = @_;
    my ($vhost);
    foreach $vhost (Cobalt::Vsite::vsite_list()) {
      if ($group eq $$vhost[2]) {
	# return $$vhost[1];
	return @$vhost;
      }
    }
    return 0;
}

sub vsite_list
# Finds all virtual web interfaces
# Arguments: none
# Return value: nested array of:
#    (ip,hostname,group,webmaster email)
#    sorted by fully qualified domain name (aka "hostname" for vsite_* functions)
# Example: @vhosts = Cobalt::Vsite::vsite_list();
#          ..OR.. 
#          foreach $vhost (@vhosts) { &use( @$vhost ); }
# Parses httpd.conf
{
    my(@vnames,$nic,$ip,$name,$adminemail,$group);
    open(HTCNF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);
    while(<HTCNF>) 
    {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/)
	{
	    $ip = $1 if (/^<VirtualHost ([\d\.]+)>/);
	    $name = $1 if (/^ServerName ([a-z\d\.\-]+)/);
	    $adminemail = $1 if (/^ServerAdmin ([\w\@\-\.]+)/);
	    $group = $1 if (/^DocumentRoot $Userdir\/([^\/]+)/o);

	    if ($group && /^<\/VirtualHost>$/) {
		chomp($group);
		push( @vnames, ([$ip,$name,$group,$adminemail]) ); 
		$ip = $name = $adminemail = $group = '';
	    }
	}
    }
    close(HTCNF);

    # sort by hostname
    @vnames=sort { ${ $a }[ 1 ] cmp ${ $b }[ 1 ] } @vnames;

    return (@vnames);
}

sub vsite_admin_list
# Lists the site admins for the given site
# Arguments: site group name
#   With no arguments, lists *all* site administrators
# Return value: the list of admins for that site
# Side effects: none
# Example: (@admins) = Cobalt::Vsite::vsite_admin_list("eth1:12");
{
    my $group = shift;
    my $qstr;

    if ($group) {
	$qstr = qq[vsite = '$group'];
    } else {
	$qstr = qq[name <> 'default'];
    }

    return (Cobalt::Meta->SQL( qq[SELECT name FROM users WHERE admin IS TRUE AND $qstr] ));
}

sub vsite_admin_set
# Set the list of site administrators for a virtual site
# Arguments: site group, list of users
# Side effects: sets the membership of the group for that site
#   to the given list, adds the site admins to the global
#   site admins group
# Return value: the usual status string of the form "3667 You are in
#   a dark place.  You are likely to be eaten by a grue."
{
    my ($group,@admins) = @_;
    my @current_admins = Cobalt::Group::group_show($group);
    Cobalt::Group::group_del_user($Sites_AdminGroup,@current_admins);

    Cobalt::Vsite::vsite_get_siteadmin_ftphelp(@current_admins);
    
    my $ret = Cobalt::Group::group_add_user($Sites_AdminGroup,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    $ret = Cobalt::Group::group_set_user($group,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    Cobalt::Vsite::vsite_set_siteadmin_ftphelp($group,@admins);
    return MSG_ok("vsite_admin_set_ok");
}

sub vsite_admin_add
# Add a site administrator to a virtual site
# Arguments: site group, list of users to add
# Side effects: adds the user(s) to the group for that site
#   and to the global site admins group
# Return value: the usual status string of the form MSG_error("4210")
{
    my ($group,@admins) = @_;

    my $ret;

    $ret = Cobalt::Group::group_add_user($Sites_AdminGroup,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group
    
    $ret = Cobalt::Group::group_add_user($group,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group
    
    # Add assorted $FtpAdminHelp files
    Cobalt::Vsite::vsite_set_siteadmin_ftphelp($group,@admins);

    return MSG_ok("vsite_admin_add_ok");
}

sub vsite_admin_del
# Remove a site administrator from a virtual site
# Arguments: site group, list of users to remove
#   from the admin list
# Side effects: removes the user(s) from the group for that site
#   and the global site admins group
# Return value: the usual status string of the form "2600 Game over, insert
#   coin"
# Example: Cobalt::Vsite::vsite_admin_del("eth10:1",@doomedusers);
{
    my ($group,@admins) = @_;
    my $ret = Cobalt::Group::group_del_user($Sites_AdminGroup,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    $ret = Cobalt::Group::group_del_user($group,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    # Delete assorted $FtpAdminHelp files
    Cobalt::Vsite::vsite_get_siteadmin_ftphelp(@admins);

    return MSG_ok("vsite_admin_del_ok");
}

sub vsite_list_alias
## list alias names for a vhost
# Arguments : site group name (if NULL, query for ALL web server aliases)
# Returns   : array of hostnames
# Examples  : vsite_list_alias($group);
{
    my $group = shift;
    my $ret   = '';
    my @list  = ();

    # read the current configuration
    open(HTCONF,"$Htconf/httpd.conf") ||
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    if ($group) {
	while (<HTCONF>) {
	    if (m|^DocumentRoot $Userdir/$group/$Webdir$|) {
		while (<HTCONF>) {
		    last if (m|^</VirtualHost>|i);
		    if (m|^ServerAlias\s+(.+)$|i) {
			push @list, split /\s+/, $1;
		    }
		}
	    }
	}
    }
    else {
	while (<HTCONF>) {
	    if (m|^ServerAlias (.+)$|i) {
		push @list, split /\s+/, $1;
	    }
	}
    }

    close HTCONF;
    return @list;
}

sub vsite_set_alias
# Set alias names for a vhost
# Arguments : site group name, list of aliases
# Returns   : string of the form MSG_error("3311")
# Notes     : replaces ALL server aliases for this site with 
# Example   : vsite_set_alias($group, $alias0, $alias1);
{
    my ($group, @aliases) = @_;
    my $ret;

    return MSG_error("vsite_invalid_noentry")
	unless $group;

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open(HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    while (<HTCONF>) {
	if (m|^DocumentRoot $Userdir/$group/$Webdir$|) {
	    print HTCTMP;
	    print HTCTMP "ServerAlias ", join(" ", @aliases), "\n" if scalar(@aliases);
	    while (<HTCONF>) {
		next if (m|^ServerAlias|i);
		print HTCTMP;
		last if (m|^</VirtualHost>|i);
	    }
	}
	else { print HTCTMP }
    }

    close HTCONF;
    close HTCTMP;

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_alias");
}

sub vsite_add_alias
# Set alias names for a vhost
# Arguments: site group name, list of aliases
# Return value: string of the form MSG_error("3311")
# Note: does not remove any old server aliases
# Example: vsite_add_alias($group, $alias0, $alias1);
{
    my ($group, @aliases) = @_;
    my $ret;
    my $found=0;

    return MSG_error("vsite_invalid_noentry")
	unless ($group && scalar(@aliases));

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open(HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    $found = 0;
    while (<HTCONF>) {
	if (!$found && m|^DocumentRoot $Userdir/$group/$Webdir$|) {
	    print HTCTMP;
	    print HTCTMP "ServerAlias ", join(" ", @aliases), "\n";
	    $found = 1;
	}
	else { print HTCTMP }
    }

    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $group)) unless ($found);
    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_alias");
}

sub vsite_del_alias
### Delete alias names for a vhost
# Arguments : site group name, list of aliases
# Returns   : string of the form MSG_error("3311")
# Note      : does not remove any old server aliases
# Example   : vsite_del_alias($group, $alias0, $alias1);
{
    my ($group, @aliases) = @_;
    my $ret;

    return MSG_error("vsite_invalid_noentry")
	unless ($group && scalar(@aliases));

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open(HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    my %ahash = map { $_ => 1; } @aliases;
    while (<HTCONF>)
    {
	if (m|^DocumentRoot $Userdir/$group/$Webdir$|) {
	    print HTCTMP;
	    while (<HTCONF>)
	    {
		if (m|^ServerAlias ([a-z\d\.\-]+)|i) {
		    print HTCTMP unless (exists($ahash{$1}));
		}
		else {
		    print HTCTMP;
		    last if (m|^</VirtualHost>|);
		}
	    }
	}
	else { print HTCTMP }
    }

    close HTCONF;
    close HTCTMP;

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_alias");
}

sub vsite_set_php
# Enable or disable the use of php4 for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form MSG_error("4400")
# Side effects: modifies the virtual host entry in httpd.conf
# Example: Cobalt::Vsite::vsite_set_ssi("home",1);
{
    my ($vsite,$php_enabled) = @_;
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    my $php_was = "";
    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		warn "vsite_set_php found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;
		    if ($php_enabled) {
			print HTCTMP "AddType application/x-httpd-php .php4\n";
			print HTCTMP "AddType application/x-httpd-php .php\n";
			$on = MSG_get("On_string");
		    }
		    else {
			$on = MSG_get("Off_string");
		    }
		} else {
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^AddType application\/x-httpd-php\d* .php/) {
		    # clear the existing AddHandler .shtml entries
		    warn "vsite_set_php found AddType .php\n" if ($Debug);
		    $php4_was = "on";
		} elsif (/^AddType application\/x-httpd-php\d*-source .php/o) {
		    # clear the existing AddType .shtml entries
		    warn "vsite_set_php found AddType .phps\n" if ($Debug);
		} else {
		    # within vhost section but not a php line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
	    }
	} else {
	    # outside vhost section! spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);
    return vsite_abort(MSG_ok("vsite_ok_php"))
	if ($php_was eq $php_enabled);

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_php");
}

sub vsite_set_ssi
# Enable or disable the use of SSIs for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form MSG_error("4400")
# Side effects: modifies the virtual host entry in httpd.conf
# Example: Cobalt::Vsite::vsite_set_ssi("home",1);
{
    my ($vsite,$ssi_enabled) = @_;
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    my $ssi_was = "";
    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		warn "vsite_set_ssi found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;
		    if ($ssi_enabled) {
			print HTCTMP "AddHandler server-parsed .shtml\n";
			print HTCTMP "AddType    text/html     .shtml\n";
			$on = MSG_get("On_string");
		    }
		    else {
			# httpd.conf gets big enough without this bloat /wdh
			# print HTCTMP "#AddHandler server-parsed .shtml\n";
			# print HTCTMP "#AddType    text/html     .shtml\n";
			$on = MSG_get("Off_string");
		    }
		} else {
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^AddHandler server-parsed .shtml/) {
		    # clear the existing AddHandler .shtml entries
		    warn "vsite_set_ssi found AddHandler .shtml\n" if ($Debug);
		    $ssi_was = "on";
		} elsif (/^\#AddHandler server-parsed .shtml/) {
		    # clear the existing AddHandler .shtml entries
		    warn "vsite_set_ssi found AddHandler .shtml\n" if ($Debug);
		    $ssi_was = "";
		} elsif (/^(\#)?AddType\s+text\/html\s+.shtml/o) {
		    # clear the existing AddType .shtml entries
		    warn "vsite_set_ssi found AddType .shtml\n" if ($Debug);
		} else {
		    # within vhost section but not a ssi line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
	    }
	} else {
	    # outside vhost section! spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);
    return vsite_abort(MSG_ok("vsite_ok_ssi_nochange"))
	if ($ssi_was eq $ssi_enabled);

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_ssi", $on);
}

sub vsite_get_php
# Get the current PHP4 settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vsite::vsite_get_php4("home");
{
    my ($vsite) = @_;
    my ($found_web);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^AddType application\/x-httpd-php\d* .php/) {
		    close HTCONF;
		    return 1;
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return 0;
}

sub vsite_get_ssi
# Get the current SSI settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vsite::vsite_get_ssi("home");
{
    my ($vsite) = @_;
    my ($found_web);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^AddHandler server-parsed .shtml/) {
		    close HTCONF;
		    return 1;
		} elsif (/^\#AddHandler server-parsed .shtml/) {
		    close HTCONF;
		    return 0;
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return 0;
}

sub vsite_set_casp
# Enable or disable the use of ASPs for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form MSG_error("4400")
# Side effects: modifies the virtual host entry in httpd.conf
# Example: Cobalt::Vsite::vsite_set_asp("home",1);
{
    my ($vsite,$casp_enabled, $nostop) = @_;
 
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    my $casp_othersite_use = 0;
    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		warn "vsite_set_casp found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;
		    if ($casp_enabled) {
			print HTCTMP "# AddHandler chiliasp .asp\n";
			print HTCTMP "# AddHandler chiliasp .asa\n";
			if ($vsite eq $MainGroup) {
			    print HTCTMP "Alias /caspclient $ASP_home/caspsamp/friendship/client/content\n";
			    print HTCTMP "Alias /caspagent $ASP_home/caspsamp/friendship/agent/content\n";
			    print HTCTMP "Alias /casp401k $ASP_home/caspsamp/401K/content\n";
			    print HTCTMP "Alias /caspsamp $ASP_home/caspsamp\n";
			    print HTCTMP "Alias /caspdoc $ASP_home/caspdoc\n";
			}
			$on = MSG_get("On_string");
		    }
		    else {
			$on = MSG_get("Off_string");
		    }
		} else {
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^# AddHandler chiliasp .asp/) {
		    warn "vsite_set_casp found AddHandler .asp\n" if ($Debug);
		    $ssi_was = "on";
		} elsif (/^# AddHandler chiliasp .asa/) {
		    warn "vsite_set_casp found AddHandler .asa\n" if ($Debug);
		    $ssi_was = "";
		} elsif (($vsite eq $MainGroup) and /^Alias \/casp/) {
		    warn "vsite_set_casp found casp Alias\n" if ($Debug);
		} else {
		    # within vhost section but not a casp line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
                $casp_othersite_use = 1 if (/chiliasp/);
	    }
	} else {
	    # outside the required vhost section! spit it..		
	    $casp_othersite_use = 1 if (/chiliasp/);
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);
    return $ret if (($ret = vsite_commit()) !~ /^2/);

    my $site_fqdn = (Cobalt::Vsite::vsite_get_bygroup("$vsite"))[1];
    my $enable = $casp_enabled ? 'yes' : 'no';
    `$ASP_vhostctrl engine=$ASP_home/$ASP_servicename vhost=$site_fqdn enable=$enable`;

    # Automatically start/stop ASP services if we're the first/last site to
    # parse or not parse ASP. we don't need to restart the casp daemon
    # anymore.
    if ($casp_enabled) {
        Cobalt::Services::set_casp_server_on(1) unless Cobalt::Services::get_casp_server_on();
    } elsif (!$casp_othersite_use) {
        Cobalt::Services::set_casp_server_on(0) unless $nostop;
    }
    return MSG_ok("vsite_ok_casp", $on);
}

sub vsite_get_casp
# Get the current ASP settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vsite::vsite_get_ssi("home");
{
    my ($vsite) = @_;
    my ($found_web);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^# AddHandler chiliasp/) {
		    close HTCONF;
		    return 1;
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return 0;
}


sub vsite_get_casp_apps
{
    my @apps;
    open(ASPCONF, "$Userdir/$vsite/$Webdir/$ASP_appconf") or return 0;
    
    while (<ASPCONF>) {
	next unless /^\[applications\]/;
	last;
    }
    while (<ASPCONF>) {
	next unless /^\//;
	$_ =~ /^\/*(.*)/;
	push @apps, $1;
    }
    close(ASPCONF);

    return @apps;
}

# note: we don't actually delete old apps as they might return
sub vsite_set_casp_apps
{
    my ($vsite, @apps) = @_;
    my $app, $newapplist = ',' . join(',', @apps) . ',';
    my $webdir = "$Userdir/$vsite/$Webdir";

    # see what we currently have 
    my $oldapplist = ',' . join(',', vsite_get_casp_apps) . ',';

    open(ASPCONF, ">$webdir/$ASP_appconf") or return 0;
    print ASPCONF "[applications]\n";
    foreach $app (@apps) {
	$app = $1 if ($app =~ /^\/*(.*)/);
	print ASPCONF "/$app\n";
	unless (($oldapplist =~ /,$app,/) or (-d "$webdir/$app")) {
	    mkdir("$webdir/$app", 02775);
	    chown((getpwnam('nobody'))[2], (getgrnam($vsite))[2],
		  "$webdir/$app");
	}
    }
    close(ASPCONF);
    return 1;
}

sub vsite_set_cgis
# Enable or disable the use of CGIs for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form "8750 I can't hear you, 
#   there's a bannana in my ear!"
# Side effects: modifies the virtual host entry in httpd.conf.
# Example: Cobalt::Vsite::vsite_set_cgis("eth0:0",1);
{
    my ($vsite,$cgis_enabled) = @_;
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		warn "vsite_set_cgis found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;
		    # Put in the correct new AddHandler entries
		    if ($cgis_enabled) {
			print HTCTMP "AddHandler cgi-wrapper .cgi\n";
			print HTCTMP "AddHandler cgi-wrapper .pl\n";
			$on = MSG_get("On_string");
		    }
		    else {
			# httpd.conf gets big enough without this bloat /wdh
			# print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
			# print HTCTMP "#AddHandler cgi-wrapper .pl\n";
			$on = MSG_get("Off_string");
		    }
		} else {
		    # end of vhost section but not the right one! spit it..
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^(\#)?AddHandler cgi-wrapper/) {
		    # clear the existing AddHandler entries
		    warn "vsite_set_cgis found AddHandler\n" if ($Debug);
		} else {
		    # within vhost section but not a cgi line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
	    }
	} else {
	    # outside vhost section! spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("2144", $on);
}

sub vsite_get_cgis
# Get the current CGI settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vsite::vsite_get_cgis("eth0:0");
{
    my ($vsite) = @_;
    my ($found_web);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^AddHandler cgi-wrapper/) {
		    close HTCONF;
		    return 1;
		} elsif (/^\#AddHandler cgi-wrapper/) {
		    close HTCONF;
		    return 0;
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return 0;
}

sub vsite_getgroup
# Given a user name, figure out what site they are associated with
# Arguments: user name
# Return value: site group name
# Side effects: none
{
    my $user = shift;
    return unless ($user);
    return Cobalt::Meta->SQL( qq[SELECT DISTINCT vsite FROM users WHERE name = '$user'] );
}

sub inform_BigBrother
# If $Dns_notification is readable and defined with an email address,
# send snitch-mail of any activity that requires an informed 3rd party
#
# Feature added by SimpleNet request 08/98 /wdh
#
# Arguments: subject string, text body string
{
    my($subject,$body) = @_;
    my($notifaddr);

    if (-r $Dns_notification)
    {
	open(NOTIF,$Dns_notification) || return 0;
	chomp($notifaddr = <NOTIF>);
	close(NOTIF);

	# recipient sanity check
	return 0 if ($notifaddr !~ /^\w+\@\w+/);

	open(MALR,"| $Sendmail -oi -t") || return 0;
	print MALR <<EOF;
To: $notifaddr
From: admin
Subject: $subject

$body

EOF
        close(MALR);
    }

    # Intended to run without $Dns_notification set as well
    return 1;
}

sub vsite_set_siteadmin_ftphelp
{
    my($group,@admins) = @_;

    # Add site admin ftp instructions
    if ($Features{'proftpd'})
    {
	if (-r $FtpAdminHelpSrc)
	{
	    my($ftphelpfile);
	    open(FTPH,"$FtpAdminHelpSrc") || return;
	    while(<FTPH>) { $ftphelpfile .= $_; }
	    close(FTPH);

	    my($siteadmin,$adminroot);
	    foreach $siteadmin (@admins)
	    {
		$adminroot = (getpwnam($siteadmin))[7];
		unless (-e "$adminroot/$FtpAdminHelp")
		{
		    open(FHLP,"> $adminroot/.ftphelp") || return;
		    print FHLP $ftphelpfile;
		    close(FHLP);
		    chown( (getpwnam($siteadmin))[2], (getgrnam($group))[2],
			   "$adminroot/$FtpAdminHelp");
		    chmod(0664,"$adminroot/$FtpAdminHelp");
		}
	    }
	}
    }
}

sub vsite_get_siteadmin_ftphelp
{
    my(@admins) = @_;
    
    # Remove site admin ftp instructions
    if ($Features{'proftpd'})
    {
	my($siteadmin,$adminroot);
	foreach $siteadmin (@admins)
	{
	    $adminroot = (getpwnam($siteadmin))[7];
	    unlink("$adminroot/$FtpAdminHelp")
		if (-e "$adminroot/$FtpAdminHelp");
	}
    }
}

sub vsite_set_fpx
# Enable or disable the use of FPX allowOverride permissions for 
# virtual sites
#
# Arguments: site group name, true/false to enable/disable, 
# Return value: status string of the form MSG_error("MS slop incapable")  
# Side effects: creates/modifies Directory blocks in access.conf
# Example: $ret=vsite_set_fpx($group,1,@userwebs);
{
    my ($vsite, $status, @userwebs) = @_;
    my ($inentry, $found_web, $conf);
    my ($on,$ret);
    # Uniquify arguments
    my(%userwebs) = map { $_, 1 } @userwebs;
    @userwebs = keys ( %userwebs );

    # verify and grab our locks

    # access.conf
    return(MSG_error("vsite_cant_locked"))
        if (-e "$Lockdir/access.conf");

    if (@userwebs)
    {
        # httpd.conf
        open(HTCONF,"$Htconf/httpd.conf") ||
            return MSG_error("vsite_cant_read_httpd", $Htconf);

        return $ret if (($ret = vsite_lock()) !~ /^2/);
        open (HTCTMP, ">$Lockdir/httpd.conf") ||
            return MSG_error("vsite_wicked");

        # httpd.conf first.  User webs require three ScriptAlias's per web.  
        #                    Site webs require none, mod_frontapge handles it.
        $found_web = $inentry = 0;
        # fix the httpd.conf file
        while (<HTCONF>) {
            if (!$inentry and
                /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {

                if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
                    $found_web = 1;
                    warn "vsite_set_fpx found $vsite\n" if ($Debug);
                }

                if (/^<\/VirtualHost>$/)
                {
                    if ($found_web) {
                        print HTCTMP $conf;
                        $inentry = 1;

                        # Put in the correct new ScriptAlias entries
                        if ($status) {
                            # print HTCTMP "ScriptAlias /_vti_bin/_vti_adm/ $Userdir/$vsite/$Webdir/_vti_bin/_vti_adm/\n";
                            # print HTCTMP "ScriptAlias /_vti_bin/_vti_aut/ $Userdir/$vsite/$Webdir/_vti_bin/_vti_aut/\n";
                            # print HTCTMP "ScriptAlias /_vti_bin/ $Userdir/$vsite/$Webdir/_vti_bin/\n";
    
                            # User webs
                            my($userInstance);
                            foreach $userInstance (@userwebs)
                            {
                                unless($userInstance eq $vsite)
                                {
                                    print HTCTMP "ScriptAlias /~$userInstance/_vti_bin/_vti_adm/ $Userdir/$vsite/users/$userInstance/$Webdir/_vti_bin/_vti_adm/\n";
                                    print HTCTMP "ScriptAlias /~$userInstance/_vti_bin/_vti_aut/ $Userdir/$vsite/users/$userInstance/$Webdir/_vti_bin/_vti_aut/\n";
                                    print HTCTMP "ScriptAlias /~$userInstance/_vti_bin/ $Userdir/$vsite/users/$userInstance/$Webdir/_vti_bin/\n";
                                }
                            }
                            $on = MSG_get("On_string");
                        }
                        else {
                            $on = MSG_get("Off_string");
                        }
                        print HTCTMP <<EOF;
AliasMatch ^/~([^/]+)(/(.*))? $Userdir/$vsite/users/\$1/web/\$3
EOF
                    } else {
                        # end of vhost section but not the right one! spit it..
                        print HTCTMP $conf;
                    }
                    print HTCTMP;
                    $conf = '';
                } elsif ($found_web) {
    
                    if (/^ScriptAlias / && /\/_vti_bin/) {
                        # clear the existing ScriptAlias entries
                        warn "vsite_set_fpx found FPX ScriptAlias\n" if ($Debug);
                    } elsif (/^AliasMatch \^\/~/) {
                        warn "found aliasmatch and web\n" if ($Debug);
                    } else {
                        # within vhost section but not an fpx line! slurp!
                        $conf .= $_;
                    }
                } else {
                    # within vhost section but not the right one! slurp!
                    $conf .= $_;
                }
            } else {
                # outside vhost section! spit it..
                print HTCTMP;
            }
        }
        close HTCONF;
        close HTCTMP;

        vsite_commit();
        # That's the last of the httpd.conf mods
    }

    # We only AllowOverride All and Options All sites that have frontpage enabled.
    # You're only shooting yourself in the foot if you're into that sort of thing.

    # Track first/last site FP usage to tweak systee
    # security.  No need to have suid 0 unless we're using it
    my($other_vsite_use) = 0;

    # update the current access configuration
    open(HTAX,"$Htconf/access.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    open (HTCTMP, ">$Lockdir/access.conf") ||
	return MSG_error("vsite_wicked");

    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTAX>) 
        {
	if (/^<Directory $Userdir\/$vsite/ ... /^<\/Directory/)
	    {
	    $found_web = 1;
            print HTCTMP if ($status);
	    next;
	    }
        else
            {
            $other_vsite_use = 1 if (/AllowOverride All/);
            }
	print HTCTMP;
        }
    close (HTAX);

    if (!$found_web && $status) 
	{
	print HTCTMP <<EOF;
<Directory $Userdir/$vsite>
AllowOverride All
Options All
</Directory>
EOF
	}
    close (HTCTMP);

    # silly
    $on = MSG_get("On_string") if ($status);
    $on = MSG_get("Off_string") unless ($status);

    rename("$Lockdir/access.conf", "$Htconf/access.conf") ||
        return vsite_abort(MSG_error("4585"));
    
    # That's it for access.conf mods

    # Enable/disable suid 0 access 
    if ($status)
        {
        chmod(04755, $Fpx_fpexec);
        chmod(00755, '/usr/sbin/suexec');
        }
    elsif (!$other_vsite_use)
        {
        chmod(00755, $Fpx_fpexec);
        chmod(04755, '/usr/sbin/suexec');
        }

## commented out by duncan,
## this is done in site Add/Mod and user Add/Mod
## (or at least it should be)
#    Cobalt::Http::http_restart();
    
    return MSG_ok("vsite_ok_fpxset", $on); # Uses $on
}

sub vsite_get_fpx
# Get the current FPX settings for a virtual site
# Arguments: site group name
# Return value: array of user webs if enabled
#               null if fpx disabled
# Side effects: parses httpd.conf
# Example: Cobalt::Vsite::vsite_get_fpx($MainGroup);
{
    my ($vsite) = @_;
    my ($found_web);
    my(@ret,%beenthere); # init return value, singularity check

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") ||
        return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
        if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {

            $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
            if ($found_web) {
                if (/^ScriptAlias\s+\/([^\/]+)\// && /_vti_bin/) {
                    my $user;

                    $user = $1 if (/^ScriptAlias\s+\/([^\/]+)\//); # user web
                    if ($user =~ /^\~/)
                    {
                        $user =~ s/^\~//; # Strip ~ from ~user
                        push(@ret,$user) unless ($beenthere{$user}); # user web
                        $beenthere{$user}=1;
                    } else
                    {
                        # $user eq _vti_bin, check algorithm
                        push(@ret,$vsite) unless ($beenthere{$vsite});
                        $beenthere{$vsite}=1;
                    }
                }

                last if (/^<\/VirtualHost>$/);
            }
        }
    }
    close HTCONF;
    return @ret;
}

sub vsite_sync
# Updates virtualhost configuration to match non-virtual network configuration
# Critical for Netif-bound web url rewrites, sharing the Netif ip address
# Arguments: none
# Return value: true for success, false for failure
{
    my $flag = shift;
    # Obtain stale configuration to remove symlinks, etc..
    my($origip,$orighost,$origgroup,$webmaster) = vsite_get_bygroup($MainGroup);

    # Fetch old site configuration, to be re-added
    my($fpx_state)=fpx_get_web($MainGroup);
    my(@fpx_webs)=vsite_get_fpx($MainGroup) if ($fpx_state);
    my($site_ssi)=vsite_get_ssi($MainGroup);
    my($site_php)=vsite_get_php($MainGroup);
    my($site_cgi)=vsite_get_cgis($MainGroup);
    my($ftpAnon,$ftpMaxUsers,$ftpQuota)=Cobalt::Ftp::ftp_get_anonymous($MainGroup);
    my ($site_casp) = vsite_get_casp($MainGroup);
    vsite_set_casp($MainGroup, 0, 'nostop') if $site_casp;

    vsite_del($MainGroup) if ($origip); # Return value insignificant

    # Symlink management to facilitate /siteadmin/ redirects
    unlink("$AdminPages_dir/siteManage/$orighost")
	if (-l "$AdminPages_dir/siteManage/$orighost");
    unlink("$AdminPages_dir/siteManage/$origip")
	if (-l "$AdminPages_dir/siteManage/$origip");
    
    # Determine our real net settings
    my $hostname = Cobalt::Network::net_get_hostname();

    # The first time the system is booted it has no hostname; the name
    # defaults to "localhost".  If the main web server is configured as
    # a virtual host, and you try to access the system by name
    # before the system knows its own name, the web server returns a
    # "unknown host" error.  So, in the special case that we don't have
    # a real hostname, we don't add the virtual host entry.
    return 1 if ($hostname eq "localhost");

    my $domainname = (Cobalt::Network::net_get_dnsclient())[0];
    my $fqdn = "$hostname.$domainname";

    my($ip,$netmask,$mac)=Cobalt::Network::net_get_ipaddr();

    # Migrate virtual sites
    if ($ip ne $origip)
    {
      # Delete the superflous virtual network interface that results from
      # changing a real network interface's IP address to one that was in
      # use as a virtual network interface.
      my(@all_sites)=vsite_list();
      my($site_instance);
      foreach $site_instance (@all_sites)
      {
        net_del_vinterface( $ip ) if ( @$site_instance[0] eq $ip );
        # We ignore the return value as $ip could already be in use by a
        # real network interface
      }
      vsite_migrate_ipaddr($origip,$ip); 
    }

    # Re-enable the Netif's vsite
    my($return)=vsite_add($ip,$fqdn,$MainGroup,$webmaster,$netmask);
    return 0 if ($return !~ /^2/);

    # Restore site services (httpd configuration only)
    # Do not return on a failed service configuration, move on to the next
    vsite_set_fpx($MainGroup,1,@fpx_webs) if ($fpx_state);
    rename("$Fpx_cnfdir/$orighost:80.cnf", "$Fpx_cnfdir/$fqdn:80.cnf");

    vsite_set_ssi($MainGroup,1) if ($site_ssi);
    vsite_set_php($MainGroup,1) if ($site_php);
    vsite_set_cgis($MainGroup,1) if ($site_cgi);
    vsite_set_casp($MainGroup,1) if $site_casp;
    my($return)=Cobalt::Ftp::ftp_config($ip,1,$ftpAnon,$MainGroup,$ftpMaxUsers,$ftpQuota);

    # Recreate symlinks
    symlink("$AdminPages_dir/siteManage/$MainGroup",
	    "$AdminPages_dir/siteManage/$fqdn");
    symlink("$AdminPages_dir/siteManage/$MainGroup",
	    "$AdminPages_dir/siteManage/$ip");

    Cobalt::Http::http_restart() unless ($flag =~ /nohup/);

    return 0 if ($return !~ /^2/);
    return 1;
}

sub vsite_migrate_ipaddr
# Moves virtual sites from an old IP address to a new one en mass
# Useful when changing the IP of a real network interface
#
# Does not restart daemons
#
# Arguments: Old IP address, New IP address
# Return value: 1 for success, 0 for failure
{
    my($oldip,$ip)=@_;

    sleep 1 if (-e "$Lockdir/httpd.conf"); # Avoid race
    return 0 if (vsite_lock() !~ /^2/);

	# for first setup (just zero will replace other conf value w/ $ip)
	$oldip = "0.0.0.0" unless($oldip);
    # update the current configuration
    open(HTCNF,"$Htconf/httpd.conf") || return 0;

    open (HTLOCK, ">$Lockdir/httpd.conf") ||
	return vsite_abort(MSG_error("vsite_wicked"));
    while(<HTCNF>)
    {
	# subsitute the ip in apache <.> type directives
	s/(\s+)$oldip(>*)$/$1$ip$2/;
	# substitute the ip in the rewritecond..
	s/^(RewriteCond\s*!\^)$oldip\(:80\)\?\$$/$1$ip\$/;
	print HTLOCK;
    }
    close(HTLOCK);
    close(HTCNF);
    return 0 if (vsite_commit() !~ /^2/);
    return 1;
}

sub vsite_set_logrotate
# Defines logrotate entries in /etc/logrotate.d and /etc/logrotate.onboot
# for individual site log access fragments (/home/sites/<group>/logs/access
# Arguments: Site group, Rotate size in megabytes
#            If log rotate size is set to zero, disable logrotations
# Return Value: Return code and descriptive string
# Example: $return=Cobalt::Vsite::vsite_set_logrotate($MainGroup,5)
{
    my($group,$size)=@_;

    unless($size) # Disable if asked
    {
	unlink("$Logrotate_dir/$group");
	unlink("$Logrotate_boot/$group");
	return MSG_ok("vsite_ok_nologrotate");
    }

    # Minor sanity check if going to enable rotates
    return MSG_error("vsite_cant_nogroup") unless (-e "$Htgroup_dir/$group/$Logdir");

    $size.='M';
    my($rotate)=<<EOF;
$Htgroup_dir/$group/$Logdir/mail.log {
   missingok
   compress
   size $size
}

$Htgroup_dir/$group/$Logdir/ftp.log {
   missingok
   compress
   size $size
}

$Htgroup_dir/$group/$Logdir/web.log {
   missingok
   compress
   size $size
}
EOF

    open(DOTD,"> $Logrotate_dir/$group") ||
	return MSG_error("vsite_cant_writerotate");
    print DOTD $rotate;
    close(DOTD);

    open(BOOT,"> $Logrotate_boot/$group") ||
	return MSG_error("vsite_cant_writerotate");
    print BOOT $rotate;
    close(BOOT);

    return MSG_ok("vsite_ok_logrotate");
}

sub vsite_lock
# lock the httpd.conf file
# Arguments: none
# Return value: a nicely stated message string
# Side effects: trix for kids!
{
    # verify and grab our lock
    return(MSG_error("vsite_cant_locked"))
	if (-e "$Lockdir/httpd.conf");

    # Lock the file by creating a temporary file in lockdir
    open(LOCK,"> $Lockdir/httpd.conf") || 
	return(MSG_error("4593"));
    close LOCK;

    return MSG_ok("2421", $Htconf);
}

sub vsite_commit
# commit to the httpd.conf file
# Arguments: none
# Return value: a nicely stated message string
# Side effects: oops! all berries!
{
    rename("$Lockdir/httpd.conf", "$Htconf/httpd.conf") ||
	return vsite_abort(MSG_error("4585"));
    return MSG_ok("2422", $Htconf);
}

sub vsite_abort
# leave in a graceful manner
# Arguments: anything under the sun!
# Return value: garbage in, garbage out.
# Side effects: unlocks httpd.conf
{
    unlink("$Lockdir/httpd.conf") if (-e "$Lockdir/httpd.conf");
    return join('',@_);
}

sub vsite_disable
# disable a virtual site
#   1. disable all virtual site users
#   2. chmod 0000 site home directory
# arguments: vsite groupname
# return value: std
{
    my $group = shift;
    my @users = vsite_get_users($group);
    foreach (@users) {
	Cobalt::User::user_disable($_);
    }
    
    my $mode = oct '0000';
    my $dir = $Userdir.'/'.$group;
    
    chmod $mode, $dir;
    
    admserv_set_access("$AdminPages_dir/siteManage/$group", 'admin');
    
    return MSG_ok("vsite_disable_ok");
}

sub vsite_enable
# enable a virtual site
#   1. re-enable all virtual site users
#   2. chmod site home directory back to normal
# arguments: vsite groupname
# return value: std
{
    my $group = shift;
    my @users = vsite_get_users($group);
    foreach (@users) {
	Cobalt::User::user_enable($_);
    }

    # make sure we don't give shell access when vsite shell is off
    my $obj=Cobalt::Meta->create("type" => "vsite");
    $obj->retrieve($group);
    if ($obj->get("shell") ne 'on') {
	my @users = vsite_get_users($group);
	foreach (@users) {
	  Cobalt::User::user_mod($_, 0, 0, 0, $Badshell);
	}
    }

    my $mode = oct '2775';
    my $dir = $Userdir.'/'.$group;

    chmod $mode, $dir;

    admserv_set_access("$AdminPages_dir/siteManage/$group", 'require', 'group', $group);
    
    return MSG_ok("vsite_enable_ok");
}

sub vsite_issuspend
# return true/false if a site is suspended
{
    my $group = shift;
    my $dir = $Userdir.'/'.$group;

    return -1 if (-l $dir and not -e readlink($dir));      

    my $m = (stat($dir))[2];
    my $mode = $m & 07777;
    return 0 if ($mode ne oct("0000"));
    return 1;
}

sub vsite_get_users
# given a vsite group query the database
# and return an array of all usernames
{
    my $group = shift;
    return unless ($group);
    return (Cobalt::Meta->SQL( qq[SELECT name FROM users WHERE vsite = '$group'] ));
}

sub vsite_ssl_get_cert_info
# Arguments: A group name.
# Returns: Refrence to a hash of subject information.
#		   Refrence to a hash of issuer information.
#          Array of ( year, monty, day ) for expiry date.
# e.g. ( $issuer, $subject @expiry) = ssl_get_cert_info
{
	my $group = shift;
	return ssl_get_cert_info("$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_set_identity
# Arguments: A group name.
#			 Number of days the certificate is valid for.
#            A country.
#            State or province name.
#			 Locality Name (City.)
#			 Organisation Name.
#            Organisation Unit.
#			 Fully Qualified Domain Name.
#			 Email address of maintainer.
# Returns: The usual. But with mustard this time.
{
	my $group = shift;
	my $days = shift;
	my @cert_info = @_;

	return ssl_set_identity($days, @cert_info, "$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_get_cert
# Arguments: A site group name.
# Returns: The base64 encoded site certificate request as a single string;
{
	my $group = shift;
	return ssl_get_cert("$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_set_cert
# Arguments: A site name.
# 		     A string that contains the full certificate in PEM format.
{
	my $group = shift;
	my $cert = shift;
	return ssl_set_cert($cert,"$Htgroup_dir/$group/$Certdir");
}

sub vsite_get_ssl
# Arguments: A site group name.
# Returns: 1 for on, 0 for off..
{
	my $group = shift;
	return 1 if (-e "$Htssldir/$group");
	return 0;
}

sub vsite_set_sslconf
# Arguments: an object
# sets the values
{
    my ($group, $ssl) = @_;

    if ($ssl) {
	open(FILE, ">$Htssldir/$group");
	close(FILE);
    } else {
	unlink("$Htssldir/$group") if (-e "$Htssldir/$group");
    }
    return;
}

sub vsite_check_cert
# Arguments: A group name.
# Returns: The usual, bartender.
{
	my $group = shift;
	return ssl_cert_check("$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_clear
# Arguments: A group name
# Returns: The usual, nah, make it a double.
{
	my $group = shift;
	return ssl_clear("$Htgroup_dir/$group/$Certdir");
}

sub vsite_get_xml
# Arguments: A Vsite name
# Returns: The site as an XML object.
#
# Only used inside this module for the purpose of dumping Vsite objs.
{
	my $group = shift;
	return unless ($group);

	my $site = new Cobalt::Meta::vsite;
	$site->retrieve($group);
	return $site->as_xml();
}

1;

