#!/usr/bin/perl -w
# usage: vacation.pl [message] [from-address]
# Copyright 1999-2000 Cobalt Networks (http://www.cobaltnet.com)

use Cobalt::Product;
use Cobalt::Locale;
use Cobalt::Util;

use DB_File;
use Fcntl qw(O_RDWR O_CREAT);

my ($message_file,$user_from) = @ARGV;
$user_from =~ /([^\@]+)\@.*/;
my $username = $1;
my ($sendto,$sender,$returnpath,$from,$replyto,$precedence);

# get locale
my $locale = Cobalt::Locale->new();
my $emailProp = $locale->getProperty("toolsModule");
if ($emailProp) {
    require "$emailProp";
}

while (<STDIN>)
{
    if    (/From:\s*(.+)/)        { $from = &extract_address($1);       }
    elsif (/Reply-To:\s*(.+)/)    { $replyto = &extract_address($1);    }
    elsif (/Sender:\s*(.+)/)      { $sender = &extract_address($1);     }
    elsif (/Return-path:\s*(.+)/) { $returnpath = &extract_address($1); }
    elsif (/Precedence:\s*(.+)/)  { $precedence = $1;                   }
}
exit if ($precedence =~ /bulk|junk/oi);

if    ($replyto) { $sendto = $replyto;     }
elsif ($from)    { $sendto = $from;        }
elsif ($sender)  { $sendto = $sender;      }
elsif ($sender)  { $sendto = $returnpath;  }
else             { exit;                   }

my %vacadb;
tie(%vacadb,'DB_File',"$Vaca_dir/$user_from.db",O_RDWR|O_CREAT,0666)
    || die "Cannot open vacation database: $!\n";
if ($vacadb{$sendto} >= ($^T - 604800)) {
    # They've been given a reply recently
    untie %vacadb;
    exit;
} else {
    $vacadb{$sendto} = $^T;
    untie %vacadb;
}

my $message_out = `cat $message_file`;
if ($emailProp = ($locale->getProperty("email_ConvertMessage"))) {
    $message_out = &$emailProp("$message_out");
}

open (OUT, "|$Sendmail -oi -t") || die "Can't open sendmail $!\n";

print OUT "From: $user_from\n";
print OUT "To: $sendto\n";
print OUT "Precedence: $subject\n";
if ($emailProp = ($locale->getProperty("email_ContentType"))) {
    print OUT "Content-Type: $emailProp\n";
}
if ($emailProp = ($locale->getProperty("email_ContentEncoding"))) {
    print OUT "Content-Transfer-Encoding: $emailProp\n";
}
print OUT "Subject: Automated reply from $user_from\n";
print OUT "\n";

print OUT "$message_out\n";
close OUT;

exit 0;

1;

# boy, this is ugly
sub extract_address
{
    my ($from) = @_;
    my ($junk,$tmp1,$tmp2);
    if ($from =~ /.*<.*>/)              # From: Foo Bar <foobar@com.edu>
    {
        ($junk,$tmp1) = split('<',$from);
        ($tmp2,$junk) = split('>',$tmp1);
        return $tmp2;
    }
    elsif ($from =~ /.*(.*)/)   # From: foobar@com.edu (Foo Bar)
    {
        ($tmp1,$junk) = split('\(',$from);
        return $tmp1;
    }
    else                                # From: foobar@com.edu [we hope]
    {
        return $from;
    }
}
